#' @include robimetabar.R
#' @import dplyr
#' @import tidyfast
#'
NULL

#' Converts a robimetabar instance into a sample x motus matrix
#'
#' @param metabar
#' @param value
#' @param values_fill
#' @param transpose
#'
#' @return
#' @export
#'
#' @examples
as_motus_matrix <- function(metabar,value = "count",
                            values_fill = 0,transpose = FALSE) {
  metabar$data %>%
    select(sample,motu,data = !! value) %>%
    dt_pivot_wider(names_from = motu,values_from = data) -> mat

  row_names <- mat$sample

  mat %>% select(-sample) %>% as.matrix() -> mat
  rownames(mat) <- row_names
  mat[is.na(mat)] <- values_fill

  if (transpose)
    t(mat)
  else
    mat
}

#' Title
#'
#' @param data
#' @param metabar
#' @param results_in
#'
#' @return
#' @export
#'
#' @examples
import_motus_matrix <- function(data,metabar,results_in) {
  metabar$data %>%
    left_join(data %>%
              as_tibble(rownames = "sample") %>%
              dt_pivot_longer(cols = -sample,
                           names_to = "motu",
                           values_to = '.__data'),
             by=c("sample","motu")) %>%
    mutate(!!results_in := .__data) %>%
    select(-.__data) -> metabar$data

  invisible(metabar)
}