

#' Make a scatter plot comparing read and MOTU couints per sample
#'
#' @param metabar
#' @param mapping
#' @param ...
#' @param data
#' @param stat
#' @param position
#' @param na.rm
#' @param show.legend
#' @param inherit.aes
#'
#' @return
#' @export
#'
#' @examples
plot_read_x_hill <- function(metabar, q = 0,
                              mapping = NULL,
                              data = NULL, stat = "identity",
                              position = "identity", ...,
                              na.rm = FALSE,
                              show.legend = NA,
                              inherit.aes = TRUE) {

  fcall <- match.call()
  fcall$metabar=NULL
  fcall$q = NULL
  fcall[[1]] <- quote(geom_point)

  gp <- eval(fcall)

  sids <- samples_ids_colname(metabar)
  cats <- categories_colname(metabar)

  left_join(metabar %>%
              samples_read_count() %>%
              rename(read_count__=read_count),
            metabar %>%
              samples_hill(q = q) %>%
              rename(hill__=hill),
            by = sids) %>%
    left_join(samples(metabar), by = sids) %>%
    filter(read_count__ > 0 & hill__ > 0) %>%
  ggplot(aes(x = read_count__,
             y = hill__,
             col = get(cats))) +
    gp +
    scale_x_log10() +
    scale_y_log10() +
    xlab("Read counts") +
    ylab(glue::glue("Diversity (q={q})"))
}

#' @export
plot_read_x_pcr_tags <- function(metabar,
                              only,
                              library_colname,
                              only_forward_tags,
                              only_reverse_tags,
                              ncol_library,
                              mapping = NULL,
                              data = NULL, stat = "identity",
                              position = "identity", ...,
                              na.rm = FALSE,
                              show.legend = NA,
                              inherit.aes = TRUE) {

  fcall <- match.call()
  fcall$metabar=NULL
  fcall$only = NULL
  fcall$library_colname = NULL
  fcall$only_forward_tags = NULL
  fcall$only_reverse_tags = NULL
  fcall$ncol_library = NULL
  fcall[[1]] <- quote(geom_point)

  forward_factor <- factor
  reverse_factor <- factor

  gp <- eval(fcall)

  data <- if (missing(only))
            metabar$data
          else {
            if (is_character(only))
              only = tibble(motu = only)
            else if (is_robimotu(only)) {
                only = ids(only)
                colnames(only) <- "motu"
            }
            else if (is.data.frame(only) && is_character(only[[1]]))
              only = tibble(motu = only[[1]])
            else
              abort("I cannot guess the only format. Must be a character or a robimutu")

            suppressWarnings(only %>% left_join(metabar$data, by = "motu"))
          }

  bys = c(id = samples_ids_colname(metabar))
  selector = c(id = "id",
               forward_tag = forward_tags_colname(metabar),
               reverse_tag = reverse_tags_colname(metabar),
               read_count__  = "read_count__",
               category    = categories_colname(metabar))

  if (!missing(library_colname))
    selector = c(selector,library = library_colname)

  data %>%
  group_by(sample) %>% rename(id=sample) %>%
  summarise(read_count__=sum(count)) %>%
  left_join(samples(metabar), by = bys) %>%
  select(selector) -> data

  if (!missing(only_forward_tags)) {
    data <- data %>% filter(forward_tag %in% only_forward_tags)
    forward_factor <- function(x) factor(x,levels=only_forward_tags)
  }

  if (!missing(only_reverse_tags)) {
    data <- data %>% filter(reverse_tag %in% only_reverse_tags)
    reverse_factor <- function(x) factor(x,levels=only_reverse_tags)
  }


  ggplot(data,aes(x   = forward_factor(forward_tag),
                  y   = reverse_factor(reverse_tag),
                  col = category,
                  cex = sqrt(read_count__))) + gp -> gg

  if (!missing(library_colname)) {
    if (missing(ncol_library))
      ncol_library <- floor(sqrt(length(unique(data$library))))
    gg <- gg +
      facet_wrap(~ library, ncol = ncol_library)
  }

  gg +
    xlab("Forward tags") +
    ylab("Reverse tags") +
  theme(axis.text.x = element_text(size=rel(0.5),angle = 90,family="mono"),
        axis.text.y = element_text(size=rel(0.5),family="mono"))

}


