clean_ref_db <- function(refdb, taxonomy, rank_threshold,
                         n = 1,
                         error_rate = 0.1,
                         quorum = 20, dist_max = 5) {


  votes <- tibble(id = refdb$id,
                  in_pools = 0,
                  bad      = 0)

  i=n

  for (i in n) {
  distances <- lcs_score(refdb$sequence[i], refdb$sequence,
    normalize = FALSE,
    similarity_mode = "distance",
    threshold = dist_max
  )

  dist_score <- cumsum(table(rep(
    distances$score,
    refdb$features$count
  )))

  dist_score_ok <- dist_score[dist_score >= quorum]

  if (length(dist_score_ok) >= 1) {
    score_lim <- as.integer(names(dist_score_ok[1]))
  } else {
    score_lim <- dist_max
  }

  in_cluster = which(!is.na(distances$score) & distances$score <= score_lim)
  cluster <- refdb[in_cluster, ]
  cluster$taxid = cluster$features$taxid
  cluster$idx = in_cluster

  votes$in_pools[in_cluster] = votes$in_pools[in_cluster] + 1

  if (length(in_cluster) == 1) next

  cluster <- cluster %>%
    left_join(., taxonomy$nodes[taxonomy$nodes$taxid %in% .$taxid, ],
      by = "taxid"
    )  %>%
    mutate(
      path = mapply(function(x, y) c(rev(x), y), path, taxid,SIMPLIFY = FALSE),
      length = mapply(length, path)
    )

  max_length <- max(cluster$length)

  paths <- t(mapply(function(x, l) c(x, rep(NA, max_length - l)),
                  cluster$path,
                  cluster$length))
  times <- cluster$features$count

  counts = integer(ncol(paths))
  taxids = integer(ncol(paths))

  for (i in seq_len(ncol(paths))) {
    tab <- table(rep(paths[,i], times))
    x <-  tab[which.max(tab)]
    counts[i] <- x
    taxids[i] <- as.integer(names(x))
  }

  tibble(
    count = counts,
    taxid = as_robitaxid(taxids)
  ) %>%
    left_join(., taxonomy$nodes[taxonomy$nodes$taxid %in% .$taxid, ], by = "taxid") %>%
  mutate(max_count = max(count), error = max_count - count, ok = error < max_count * error_rate) %>%
    filter(ok)  %>% tail(1) -> limit


  is_subclade_of(cluster,taxonomy,limit$taxid,results_in = "subclad") %>% filter(!subclad) -> bads


  votes$bad[bads$idx] = votes$bad[bads$idx] + 1
  }
  votes$ratio = 0
  votes$ratio[votes$in_pools>0] = votes$bad[votes$in_pools>0] / votes$in_pools[votes$in_pools>0]
  votes
}

# s = 0:7 * 10282
# e = c(s[-1] - 1,82260)
# p = mapply(seq.int, e,s)
# library(parallel)
# check_ref <- mclapply(p, function(i) clean_ref_db(ref,taxonomy = taxo,
#                                      rank_threshold = "",n = i,quorum = 5,error_rate = 0.05))
#

clean_ref_db <- function(refdb, taxonomy, quorum = 10,error_rate = 0.05, rank_limit="class") {

  rdata <- tibble(taxid = refdb$features$taxid,
                  count = refdb$features$count)

  urdata <- rdata %>%
    group_by(taxid) %>%
    summarise(count = sum(count),.groups = "drop") %>%
    left_join(nodes(taxonomy) %>% select(taxid,path), by="taxid") %>%
    mutate(path = mapply(c,taxid,path), lp = mapply(length, path))

  with(urdata,
       tibble(count = rep(count,lp),
              taxid = unlist(path))) %>%
        group_by(taxid) %>%
    summarise(count = sum(count),.groups = "drop") -> grdata

  tibble(id = refdb$id,
         taxid = refdb$features$taxid,
                  count = refdb$features$count,
         sequence = refdb$sequence) %>%
    left_join(urdata %>% select(-count),by = "taxid") -> rdata


  with(rdata,
       tibble(id = rep(id,lp),
              taxid = rep(taxid,lp),
              count = rep(count,lp),
              path = unlist(path),
              rank = unlist(mapply(seq,lp)),
              sequence= rep(sequence,lp))) %>%
    group_by(path) %>%
    mutate(total = sum(count)) -> grdata

  print(grdata)

  grdata %>%
    filter(total >= quorum) %>%
    group_by(id) %>%
    filter(rank == min(rank))  -> rdata

  print(rdata)

  grdata[grdata$path %in% unique(rdata$path),c("id","path","count","sequence")] %>%
    group_by(path) %>%
    summarise(ids = list(id),
              counts = list(count),
              sequences = list(sequence),.groups = "drop") %>%
    mutate(rank_theshold = !is.na(taxon_at_rank(path,
                                                taxo,
                                                !!rank_limit,
                                                results_in = "rank")$rank)) %>%
    filter(rank_theshold) -> grdata
  grdata
}

outgroup <- function(ids,sequences,counts) {
  if (length(ids) > 2) {
  t(combn(ids,2)) -> results
  colnames(results) <- c("X","Y")
  results <- as_tibble(results)

  t(combn(sequences,2)) -> seqs
  results$score = lcs_score(seqs[,1],seqs[,2],similarity_mode = "distance")$score
  results %>%
    left_join(tibble(X=ids,count_x=counts),by ="X") %>%
    left_join(tibble(Y=ids,count_y=counts),by ="Y") %>%
    mutate(count = count_x * count_y) -> results

  pval = numeric(length(ids))
  for (id in seq_along(ids)) {
    ingroup  <- results %>% filter(X == ids[id] | Y == ids[id])
    outgroup <- results %>% filter(X != ids[id] & Y != ids[id])

    X <- sample(rep(ingroup$score,ingroup$count),
                floor(0.5 * sum(ingroup$count)))

    Y <- sample(rep(outgroup$score,outgroup$count),
                                   floor(0.5 * sum(outgroup$count)))

    if (length(Y) >= 5)
    pval[id] <- wilcox.test(X,Y,
                            alternative = "greater")$p.value
    else pval[id] <- 1.0

    }
  }
  else pval=1.0
  tibble(id = ids,count = counts,pvalue = pval,sequence = sequences)
}

# Si une sequence apparait plus de k fois dans l'EMBL avec le mme taxid on les garde
# On calcule le score -> filtre les erreurs potentielles.
# Sur les erreurs potentielles nouveau score bas sur le nombre de clusters de differences

# as_tibble(t(with(x[1:100,],mapply(outgroup,ids,sequences,counts))))
