


#' Extracts best ecotag identification from metabar results
#'
#' @param object
#' @param taxonomy
#'
#' @return
#' @export
#'
#' @examples
ecotag_best_identity <- function(object,taxonomy) {
  UseMethod("ecotag_best_identity",object)
}

#' @rdname ecotag_best_identity
#' @export
ecotag_best_identity.robimotu <- function(object,taxonomy) {
  mids <- ids_colname(object)
  object$best_identity %>%
    cbind(ids(object)) %>%
    gather(key = "db",value = "identity", -all_of(mids)) %>%
    group_by_at(mids) %>%
    mutate(rank__ = order(identity,decreasing = TRUE)) %>%
    filter(rank__ <= 1) %>%
    select(-rank__) %>% left_join(
      object$best_match %>%
        cbind(ids(object)) %>%
          gather(key = "db",value = "entry", -all_of(mids)),
      by = c(mids,"db")
    ) %>% left_join(
      object$taxid_by_db %>%
        cbind(ids(object)) %>%
          gather(key = "db",value = "taxid", -all_of(mids)),
      by = c(mids,"db")
    ) %>%
    ungroup() -> e

  e$taxid <- as_robitaxid_master(e$taxid)

  robimotu(e)
}

#' @rdname ecotag_best_identity
#' @export
ecotag_best_identity.robimetabar <- function(object,taxonomy) {
  motus(object) %>%  ecotag_best_identity(taxonomy)
}