#' @importFrom Rdpack reprompt
#'
NULL

#' Generalized logaritmic function.
#'
#' \deqn{x \longmapsto 1 : \log(x) \approx x-1}
#'
#' @param x A numerical vector of values, from which the logarithm must be extracted.
#' @param q The shape parameter of the generalized logarithm.
#'
#' @return a numeric vector of logarithm values
#'
#' @references
#'    \insertRef{Tsallis:94:00}{ROBITools2}
#'
#'
#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
log_q = function(x, q = 1) {
  if (q == 1)
    log(x)
  else (x^(1 - q) - 1) / (1 - q)
}


#' Derivative of the generalized exponential function.
#'
#'
#' @references
#'    \insertRef{Tsallis:94:00}{ROBITools2}
#'
#' @author Eric Coissac
#' @export
d_log_q = function(x, q = 1) {
  x^(-q)
}

#' Generalized exponential function.
#'
#'
#' @references
#'    \insertRef{Tsallis:94:00}{ROBITools2}
#'
#' @author Eric Coissac
#' @export
exp_q = function(x, q = 1) {
  if (q == 1)
    exp(x)
  else
    (1 + (1 - q) * x)^(1 / (1 - q))
}

#' @author Eric Coissac
#' @export
d_exp_q = function(x,q=1) {
  exp_q(x, q)^q
}

#' @author Eric Coissac
#' @export
dd_exp_q = function(x, q = 1) {
  exp_q(x, q)^(2 * q - 1)
}

#' @param x
#'
#' @param q
#' @param normalize
#'
#' @author Eric Coissac
#' @export
H_q = function(x, q = 1, normalize = TRUE) {
  x = x[x>0]
  if (normalize)
    x <- x/sum(x)
  sum(x * log_q( 1 / x, q))
}

#' @author Eric Coissac
#' @export
D_q = function(x, q = 1, normalize = TRUE) {
  exp_q(H_q(x, q, normalize=normalize), q)
}
