#'
#' @author Eric Coissac
#' @export
is_subclade_of = function(data, taxonomy, parent, results_in, ...) {
  UseMethod("is_subclade_of", data)
}

#' @rdname is_subclade_of
#' @export
is_subclade_of.robitaxid <- function(data, taxonomy, parent, results_in) {
  if (missing(results_in)) {
    results_in <- paste0(
      "is_",
      make.names(scientific_name(
        parent,
        taxonomy
      ) %>%
        pull(scientific_name))
    )
  }

  bys <- "taxid"
  names(bys) <- results_in

  taxids <- as_robitaxid_master(data)
  utaxids <- unique(data)

  unodes <- nodes(taxonomy)[nodes(taxonomy)$taxid %in% utaxids,c("taxid",'path')]
  unodes$lp <- sapply(unodes$path,length)

  ancestors <- c(rep(unodes$taxid,unodes$lp)[unlist(unodes$path) == parent],
                 parent[parent %in% utaxids])

  tibble(taxid = as_robitaxid_master(data),
         !! results_in := taxids %in% ancestors)
  }


#' @rdname is_subclade_of
#' @export
is_subclade_of.numeric <- function(data, taxonomy, parent, results_in) {
  fcall <- match.call()
  fcall[[1]] <- quote(is_subclade_of)
  fcall[[2]] <- quote(as_robitaxid_master(data))
  eval(fcall)
}

#' @rdname is_subclade_of
#' @export
is_subclade_of.tbl <- function(data, taxonomy, parent, results_in, slot) {
  if (missing(results_in)) {
    results_in <- paste0(
      "is_",
      make.names(scientific_name(
        parent,
        taxonomy
      ) %>%
        pull(scientific_name))
    )
  }

  if (missing(slot)) {
    slot <- master_taxids_colname(data)
  }

  bys <- "taxid"
  names(bys) <- slot

  suppressWarnings(
    data %>% left_join(data %>%
      pull(slot) %>%
      unique() %>% na.omit() %>%
      as_robitaxid_master() %>%
      is_subclade_of(taxonomy, parent, results_in), by = bys)
  )
}

#' @rdname is_subclade_of
#' @export
is_subclade_of.robimotu <- function(data, taxonomy, parent, results_in, slot) {
  fcall <- match.call()
  fcall[[1]] <- quote(is_subclade_of)
  fcall[[2]] <- quote(as_tibble(data))
  eval(fcall) %>%
    as_robimotu()
}

#' @rdname is_subclade_of
#' @export
is_subclade_of.robimetabar <- function(data, taxonomy, parent, results_in, slot) {
  fcall <- match.call()
  fcall[[1]] <- quote(is_subclade_of)
  fcall[[2]] <- quote(motus(data))
  eval(fcall)
}