#' @include utils.R
#' @include type_robitaxid.R
#'
NULL

#' Returns the reference taxonomy of an object
#'
#' robiobjects including taxonomy can be linked to a reference taxonomy.
#' This method return this taxonomy.
#'
#' @param object the interrogated object.
#'
#' @return a robitaxonomy instance.
#'
#' @author Eric Coissac
#' @export
#'
reference_taxonomy = function(object) {
  UseMethod("reference_taxonomy", object)
}


#' @rdname reference_taxonomy
#' @export
reference_taxonomy.robitaxonomy <- function(object) {
  ref <- object$reference_taxonomy
  if (is.null(ref)) object else ref
}


#' @details For \code{\link[ROBITools2]{robitaxid}}, if no reference taxonomy
#'          are attached to an instance, then the ROBITools2 default taxonomy
#'          is returned.
#'
#' @seealso \code{\link[ROBITools2]{get_default_taxonomy}},
#'          \code{\link[ROBITools2]{set_default_taxonomy}}.
#'
#' @rdname reference_taxonomy
#' @export
#'
#' @examples
#'
#'   x <- as_robitaxid(c(1,2,2759))
#'   reference_taxonomy(x)
#'
reference_taxonomy.robitaxid = function(object) {
  ref <- attr(object,"reference_taxonomy")
  if (is.null(ref)) get_default_taxonomy() else ref
}
