#' @import foreach
#' @import doParallel
NULL

.has_doParallel <- is.element("doParallel",installed.packages())
if (.has_doParallel) require(doParallel)

#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
every_oligo <- function(n,dna = c("A","C","G","T")) {
  do.call(paste0,lapply(rev(do.call(expand.grid,rep(list(dna),n))),as.character))
  }


#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
filter_tag_homopolymere <- function(tags,n) {
  pattern <- sprintf('(.)\\1{%d}', n - 1)
  tags[-grep(pattern = pattern,
             x = tags,
             perl = TRUE)]
}

#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
filter_tag_homopolymere_min <- function(tags,n) {
  pattern <- sprintf('(.)\\1{%d}', n - 1)
  tags[grep(pattern = pattern,
             x = tags,
             perl = TRUE)]
}

#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
filter_tag_gcmax <- function(tags,n) {
  cg <- vapply(lapply(lapply(strsplit(tags, ""),
                             factor,
                             levels = c("A", "C", "G", "T")),
                      table),
               function(x) sum(x[c("C", "G")]),
               0)

  tags[cg <= n]
}

#' @author Eric Coissac <eric.coissac@metabarcoding.org>
#' @export
clique_tag <- function(tags,min_dist,min_size) {
  lt <- strsplit(tags,split = "")
  d <- function(a,b) sum(a != b)

  if (.has_doParallel && getDoParRegistered()) {
    `%dp%` <- `%dopar%`
  }
  else{
    `%dp%` <- `%do%`
  }

  kmax = c()
  for (i in seq_along(tags)) {
    j <- i
    k <- lt
    r <- k[[i]]

    while (j <= length(k)) {
        z <- foreach(a = k, .combine = c) %dp% sum(a != r)
        # z <- vapply(k, d, FUN.VALUE = 0, k[k])
        k <- k[z == 0 | z >= min_dist]
        j = j + 1
    }
    message(length(k))
    if (length(k) > length(kmax))
      kmax <- k

    if (length(kmax) >= min_size)
      return(kmax)
  }

  kmax
}
