#' @import stringr
#'
NULL


.int2nuc = c("A","C","G","T")
.nuc2int = 0:3
names(.nuc2int) = .int2nuc


#' @author Eric Coissac
#' @export
rnuc = function(n,pa=1,pc=1,pg=1,pt=1) {
  p = c(pa,pc,pg,pt)/sum(pa,pc,pg,pt)
  pc = cumsum(p)
  nuc = c("A","C","G","T")[colSums(outer(pc,runif(n),"<="))+1]
  return(nuc)
}

#' @author Eric Coissac
#' @export
nuc2int = function(dna) {
  nucs = vapply(str_split(dna,''),
                function(ns) sum(.nuc2int[ns] * 4 ^ ((length(ns)-1):0)),0)
  return(nucs)
}

#' @author Eric Coissac
#' @export
int2nuc = function(n,pb) {
  pb = pb-1
  vapply(n, function(x) paste0(.int2nuc[bitwAnd(x,(3 * 4^(pb:0))) / 4^(pb:0) + 1],collapse = ""),
         "")
}

#' @author Eric Coissac
#' @export
rkmer = function(n,k,pa=1,pc=1,pg=1,pt=1) {
  vapply(1:n, function(i)
                paste(rnuc(k,pa,pc,pg,pt),
                collapse = ""),
         "")
}

#' @author Eric Coissac
#' @export
rseq = function(k,nkmer=1) {
  kmer.count = integer(4^k)
  names(kmer.count)=int2nuc(0:(4^k-1),k)
  n = 4^k - k + 1
  seq = character(n)
  i=1
  kmer=rkmer(1,k)
  kmer.num=nuc2int(kmer)
  seq[i]= kmer
  kmer.count[kmer.num+1]= 1
  while(i < n) {
    i = i + 1
    pnuc = sample(0:3,1)
    suffix = bitwAnd(kmer.num * 4,(4^k)-1)
    o.pnuc=pnuc
    while(kmer.count[suffix + pnuc + 1]==nkmer) {
      pnuc = (pnuc+1) %% 4
      if (o.pnuc==pnuc)
      {
        return(paste(seq,collapse = ""))
      }
    }
    kmer.num = suffix + pnuc
    kmer.count[kmer.num+1]=kmer.count[kmer.num+1]+1
    seq[i] = .int2nuc[pnuc+1]
  }

  return(paste(seq,collapse = ""))
}
