#' @import readr
#' @import stringr
#' @import dplyr
#' @include type_robitag.R
#' @include type_robiuniqueid.R
#' @include robimotu.R
#'
NULL



#' Reads a ngsfilter file
#'
#' Ngsfilter file are used by the *OBITools* command ngsfilter
#' to demultiplex reads and trim the primers from the raw data.
#' They can contain metadata on the samples.
#'
#' @param filename the name of the ngsfiler file
#'
#' @return a tible
#' @export
#'
#' @examples
read_ngsfilter <- function(filename) {
  lines <- read_lines(filename)
  lines <- str_split_fixed(lines,pattern = "@",n = 2)

  lines[,1] %>%
    str_trim() %>%
    str_split_fixed(pattern = "[ \t]",n=6) -> left

  colnames(left) <- c("experiment","id","tags",
                      "forward_primer","reverse_primer",
                      "full_length")


  left %>%
    as_tibble() %>%
    separate(tags,
             into = c("forward_tag","reverse_tag"),
             sep=":", remove = TRUE, fill = "right") %>%
    mutate(id = as_robiuniqueid(id),
           reverse_tag = ifelse(is.na(reverse_tag),forward_tag,reverse_tag),
           full_length = parse_logical(full_length)) %>%
    mutate(forward_tag = as_robitag_forward(ifelse(forward_tag=="-",
                                                NA,
                                                forward_tag)),
           reverse_tag = as_robitag_reverse(ifelse(reverse_tag=="-",
                                                NA,
                                                reverse_tag))) -> left

  lines[,2] %>%
    str_trim() %>%
    str_extract_all('[^ ]+ *= *([^;]+|"[^"]+|\'[^\']+)+') %>%
    lapply(str_split_fixed,pattern = " *= *",n=2) %>%
    lapply(function(x) {v <- x[,2]
                        names(v) <- x[,1]
                        v}) %>%
    feature_as_matrix() %>%
    as_tibble() -> right


  bind_cols(left,right) %>% robidata()
}
