#' @import readr
NULL

#' @export
read_obitab <- function(file,
                        delim = '\t',
                        sample = "sample",
                        sample_sep = "\\.",
                        attribute = ":",
                        motu_ids = "id",
                        sample_ids = "sample",
                        guess_max = 1000,
                        verbose = TRUE) {
  obitab <- read_delim(
    file,
    delim = "\t",
    trim_ws = TRUE,
    guess_max = guess_max,
    col_types = list(
      id = col_character() ,
      definition = col_character(),
      sequence = col_character(),
      count = col_integer()
    )
  )

  	# get the colnames matching the sample pattern

	column=colnames(obitab)
	pat = paste('(^|',sample_sep,')',sample,'[',sample_sep,attribute,']',sep='')
	scol= grep(pat,column)

	    # reads informations about samples

	reads  = obitab[,scol]
	names  = colnames(reads)
	names  = strsplit(names,split=attribute)

			# for sample name just remove the first part of the col names
			# usally "sample:"

	sample_names = sapply(names,function(a) paste(a[-1],collapse=attribute))

	reads<-t(reads)
	rownames(reads)=sample_names

	# sample's data

	sample_data <- as_tibble(do.call(rbind,strsplit(sample_names,split=attribute)))
	#rownames(sample_data) <- sample_names
	colnames(sample_data) <-strsplit(names[[1]][1],split=attribute)


	# motus information

	motus = obitab[,-scol]

	motus_id = motus$id

	#rownames(motus) <- motus_id
	colnames(reads) <- motus_id


	robimetabar(reads = reads,
	            samples = sample_data,
	            motus = motus,
	            sample_ids = sample_ids,
	            motu_ids = motu_ids,
	            verbose = verbose)


}