#' @include robimetabar.R
#' @import dplyr
#'
NULL

identical_or_na <- function(x) {
  x <- unique(x)
  if (length(x) == 1)
    x
  else
    NA
}

#' Title
#'
#' @param data
#' @param key
#'
#' @return
#' @export
#'
#' @examples
aggregate_robidata <- function(data,key) {
  id_col = ids_colname(data)
  data %>%
    mutate(`__key__` = key) %>%
    select(-!!id_col) %>%
    group_by(`__key__`) %>%
    summarise_all(identical_or_na) %>%
    mutate(!!id_col:=as_robiuniqueid(as.character(`__key__`))) %>%
    select(-`__key__`) %>%
    select(!!id_col,everything()) -> data

  sapply(data,function(c) !all(is_na(c))) -> to_keep
  data[,to_keep] %>% as_robidata()
}

samples_average_data <- function(metabar,key) {
  id_col = samples_ids_colname(metabar)
  bys <- "sample"
  names(bys) = id_col
  samples(metabar) -> s
  s$`__key__` <- as.character(key)
  s %>%
    select(!!id_col,`__key__`) %>%
    group_by(`__key__`) %>%
    mutate(`__gsize__` = length(`__key__`)) %>%
    ungroup() %>%
    right_join(metabar$data %>%
                 group_by(sample) %>%
                 mutate(`__scount__` = sum(count),
                        `__rel_freq__` = count/`__scount__`) %>%
                 ungroup(),
               by = bys) %>%
    select(-!!id_col) %>%
    group_by(`__key__`,motu) %>%
    summarise(count = sum(`__rel_freq__`)/`__gsize__`[1] * `__scount__`[1]) %>%
    select(sample=`__key__`,motu,count) %>%
    ungroup()
}

motus_average_data <- function(metabar,key) {
  id_col = motus_ids_colname(metabar)
  bys <- "motu"
  names(bys) = id_col
  motus(metabar) %>%
    mutate(`__key__` = as.character(key)) %>%
    select(!!id_col,`__key__`) %>%
    right_join(metabar$data,by = bys) %>%
    select(-!!id_col) %>%
    group_by(`__key__`,sample) %>%
    summarise_all(mean) %>%
    select(sample,motu=`__key__`,everything())
}

#' Title
#'
#' @param metabar
#' @param key
#'
#' @return
#' @export
#'
#' @examples
samples_average <- function(metabar,key) {
  data <- samples_average_data(metabar,key)
  samples <- aggregate_robidata(samples(metabar),key) %>%
    as_robisample()
  motus <- motus(metabar)

  new_robimetabar(data = data,
                  samples = samples,
                  motus = motus)
}

#' Title
#'
#' @param metabar
#' @param key
#'
#' @return
#' @export
#'
#' @examples
motus_average <- function(metabar,key) {
  data <- motus_average_data(metabar,key)
  motus <- aggregate_robidata(motus(metabar),key) %>%
    as_robimotu()
  samples <- samples(metabar)

  new_robimetabar(data = data,
                  samples = samples,
                  motus = motus)
}