#' @include robimetabar.R
#'
#' @import dplyr
#' @import openxlsx
NULL


#' @author Eric Coissac
#' @export
write_xlsx = function(x, file, asTable = FALSE, ...)  {
  UseMethod("write_xlsx",x)
}

#' @author Eric Coissac
#' @export
write_xlsx.default = openxlsx::write.xlsx

flat_tibble <-function(data) {
  as_tibble(do.call(c,lapply(data,
                             function(x) if (is.list(x)) x else list(x))))
}

#' Writes a `robimetabar` object as xlsx files.
#'
#' Save metabacoding data in excel format.
#'
#' If you encounter a memory bug run the following command
#'
#'      options(java.parameters = "- Xmx4096m")
#'
#' As the first commande of your script before importing
#' any package
#'
#' @param x
#' @param file
#' @param asTable
#' @param as_matrix
#' @param ...
#' @param transpose
#'
#' @author Eric Coissac
#' @export
write_xlsx.robimetabar = function(x, file,
                                  asTable = FALSE,
                                  as_matrix = TRUE,
                                  transpose = TRUE,
                                  ...)  {

  addsheet = function(data,sheet) {

    cn = if (inherits(data,"data.frame")) names(data) else colnames(data)

    width_vec_header <- nchar(cn)  + 2

    addDataFrame(data, sheet,
             startRow=1, startColumn=1,
             row.names = TRUE,
             colStyle = list("2"=wrapped_text),
             colnamesStyle = TABLE_COLNAMES_STYLE,
             rownamesStyle = TABLE_ROWNAMES_STYLE)

    rn = if (inherits(data,"data.frame")) row.names(data) else rownames(data)
    setColumnWidth(sheet,
                   colIndex = 1,
                   colWidth = max(nchar(rn)))


    for (i in 1:ncol(data))
      setColumnWidth(sheet,
                     colIndex = i+1,
                     colWidth = width_vec_header[i])
  }

  excel = createWorkbook()
  addWorksheet(excel,"Samples")
  addWorksheet(excel,"MOTUs")

  writeData(excel, "Samples", flat_tibble(samples(x) %>%
                               select(!!samples_ids_colname(x),
                                      everything())))
  writeData(excel, "MOTUs", flat_tibble(motus(x) %>%
                               select(!!motus_ids_colname(x),
                                      everything())) %>%
                               mutate_if(is_robitaxid,as.integer))

  if (as_matrix) {
    addWorksheet(excel,"Read counts")
    writeData(excel, "Read counts", as_motus_matrix(x, transpose = transpose),
                colNames = TRUE, rowNames = TRUE)

    x$data %>% select(-sample,-motu, -count) %>% colnames() -> extradata

    for (e in extradata) {
      addWorksheet(excel,e)
      writeData(excel, e, as_motus_matrix(x,value = e, transpose = transpose),
                colNames = TRUE, rowNames = TRUE)
    }

  }
  else {
    addWorksheet(excel,"Data")
    writeData(excel, "Data", x$data)
  }

  saveWorkbook(excel, file, overwrite = TRUE)
}

