#' @import stringr
#' @import tibble
#' @import dplyr
NULL

#' Create an instance of `robiseq`
#'
#' @author Eric Coissac
#' @export
build.robiseq = function(id,sequence,header=NULL,quality=NULL,...) {
  if (missing(id) || is.null(id))
    stop("id must be defined")

  if (missing(sequence) || is.null(sequence))
    stop("sequence must be defined")

  rep = list(...)
  rep$id=id
  rep$sequence=sequence

  if (! is.null(header))
    rep$header=header

  if (! is.null(quality))
    rep$quality=quality

  return(make.robiobject(rep,robiclass = "robiseq"))
}

.parseValue = function(x) {
  first = stringr::str_sub(x,1,1)
  last  = stringr::str_sub(x,-1,-1)

  if ((first == "{" && last == "}") ||
      (first == "[" && last == "]")) {
    x = paste0("c(",stringr::str_sub(x,2,-2),")")
    x = stringr::str_replace_all(x,":","=")
  }

  value = try(eval(parse(text=x)),silent = TRUE)

  if(inherits(value, "try-error")) {
    if (x=='True')
      value=TRUE
    else if (x=='False')
      value=FALSE
    else
      value=x
  }

  return(value)
}

#' @author Eric Coissac
#' @export
`[[.robiseq` = function(x, ...) {

  suppressWarnings(rep <- NextMethod())

  if (is.null(rep)) {
    args = list(...)
    if (length(args) > 1)
      warning("Only first argument passed to [[ will be used")

    slot = args[[1]]

    if (length(slot) > 1)
      error("Only a single slot can be returned by [[")

    # message(paste0("looking for '",slot,"'"))

    header = unmake.robiobject(x)[["header"]]

    if (is.null(header))
      value=NA
    else {
      pattern = paste0(slot,"=([^;]+);")
      value = stringr::str_match(header,pattern)[1,2]
    }

    if (! is.na(value))
      value=.parseValue(value)
  }
  else
    value=rep

  return(value)
}

#' @author Eric Coissac
#' @export
`$.robiseq` = function(x, i) {
  if (is.character(i)) {
    rep = x[[i]]
    return(rep)
  }
  error("unexpected ",class(i)," constant")
}

#' @author Eric Coissac
#' @export
`[.robiseq` = function(x, ..., drop = FALSE) {

  keys = try(list(...)[[1]],silent = TRUE)

  if(inherits(keys, "try-error")) {
    keys = character(0)
  }

  if (! is.character(keys))
    stop('Keys must be a character vector')

  keys <- c('id','sequence',setdiff(keys,c('id','sequence')))

  v <- lapply(keys, function(y) x[[y]])
  names(v) <- keys
  return(do.call(build.robiseq,v))
}

#' @author Eric Coissac
#' @export
names.robiseq = function(x) {
  rep = NextMethod()
  h = x$header
  if (!is.na(h))
    rep = union(rep,
                stringr::str_match_all(h,"([^ ]+)=[^;]+;")[[1]][,2])
  return(rep)
}

