#' @importFrom  rlang are_na
NULL

#' Title
#'
#' @param data
#'
#' @return
#' @export
#'
#' @examples
tagjump <- function(data, verbose = is_robi_verbose()) {
  UseMethod("tagjump",data)
}

#' @rdname seek_contaminents
#' @export
tagjump.robimetabar <- function(data, verbose = is_robi_verbose()) {

  data$data %>%
    left_join(samples(data),
              by = c(sample="id")) %>%
    filter_at(categories_colname(data),
              any_vars( . == "sequencing_blk")) %>%
    group_by(motu) %>%
    summarise(tagjump_max = max(count)) -> tj

  min_jmp = ceiling(mean(tj$tagjump_max))

  bys = "motu"
  names(bys) <- motus_ids_colname(data)

  motus(data) %>%
    select(motus_ids_colname(data)) %>%
    left_join(tj, by = bys) %>%
    mutate(tagjump_threshold = ifelse(are_na(tagjump_max) | tagjump_max < min_jmp,
                                      min_jmp,
                                      tagjump_max))
}


#' Title
#'
#' @param data
#'
#' @return
#' @export
#'
#' @examples
clean_tagjump <- function(data, verbose = is_robi_verbose()) {
  UseMethod("clean_tagjump",data)
}

#' @rdname seek_contaminents
#' @export
clean_tagjump.robimetabar <- function(data, verbose = is_robi_verbose()) {

  clean <- tagjump(data)

  data$data %>%
    left_join(clean,
              by = c("motu" = motus_ids_colname(data))) %>%
    mutate(tagjump_orig = count, count = ifelse(count < tagjump_threshold,
                                                  0,
                                                  count - tagjump_threshold) ) %>%
    ungroup() %>%
    filter(count > 0)
}


#' @export
hist_tag_jumps <- function(metabar,
                           maximum = FALSE,
                           mapping = NULL,
                           data = NULL, stat = "identity",
                           position = "identity", ...,
                           na.rm = FALSE,
                           show.legend = NA,
                           inherit.aes = TRUE) {

  fcall <- match.call()
  fcall$metabar = NULL
  fcall$maximum = NULL
  fcall[[1]] <- quote(geom_histogram)

  gh <- eval(fcall)

  metabar$data %>%
    left_join(samples(metabar),
              by = c(sample="id")) %>%
    filter(category == "sequencing_blk") -> data

  if (maximum)
    data %>%
      group_by(motu) %>%
      summarise(count = max(count)) -> data

  data %>% ggplot(aes(x=count)) + gh -> gg

  if(maximum)
    gg +
    xlab("Maximum read count") +
    ylab("MOTUs")
  else
    gg +
    xlab("Read count") +
    ylab("MOTU occurrences")
}


