#' @param data a metabar data strucutre
#'
#' @param verbose    if `TRUE` warnings are emitted if the constructor takes
#'                   some decision because of missing values and estimates
#'                   default values these missing data.
#'
#' @examples
#'   library(tidyverse)
#'   reads <- matrix(1:12, nrow = 3)
#'   rownames(reads) <- c("sample_1", "sample_2", "sample_3")
#'   colnames(reads) <- c("motu_1", "motu_2", "motu_3", "motu_4")
#'
#'   metabar <- robimetabar(reads = reads)
#'
#'   samples(metabar)
#'   metabar %>% reset_category_tags %>% samples
#'
#' @author Eric Coissac
#' @export
resets_category_tags <- function(data,verbose=TRUE) {
  UseMethod("resets_category_tags", data)
}

#' @rdname reset_category_tags
#' @export
resets_category_tags <- function(data,verbose=TRUE) {
  has_categories <- eval(quote("category" %in% colnames(samples)),envir = data)
  predefined <- ! get("sample_categories", envir = data)

  if ( has_categories && predefined) {
    if (verbose)
      warning("A category column was already defined for sample descriptions, and will be erased")
  }

  assign(
      x = "categories", value = factor(rep("Samples", nrow(data)),
                                       levels = c("Samples",
                                                  "Extraction controls",
                                                  "PCR controls",
                                                  "Sequencing controls",
                                                  "Positive controls")),
      envir = data)


  data
}

#' @author Eric Coissac
#' @export
tags_positive_ctrls <- function(data,verbose=TRUE) {
  UseMethod("tags_positive_ctrls", data)
}


#' @author Eric Coissac
#' @export
tags_positive_ctrls.robimetabar <- function(data,pattern,verbose=TRUE) {

  if (!has_categories || predefined)
  rep <- make_robiobject(
    get("motus", envir = data),
    c("robimotus", "robidata")
  )
  attributes(rep)$robi.ids <- get("motu_ids", envir = data)

  rep
}
