% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robimotu.R
\name{build_motu_names}
\alias{build_motu_names}
\title{Builds a serie of motu names.}
\usage{
build_motu_names(
  n,
  ndigits = max(floor(log10(n)) + 1),
  sep = "_",
  verbose = is_robi_verbose()
)
}
\arguments{
\item{n}{an \code{integer} vector indicating the number of motu names to
generate per category. If \code{n} has names, they are used as category
names. Otherwhise category names are generated by concatenating the word
\code{motu} to a serie of letters.}

\item{ndigits}{a singme integer value indicating on how many digits the motu
numerical serie number have to be padded with 0s.}

\item{sep}{a character string to separate the name parts}

\item{verbose}{if `TRUE` warnings are emitted if the constructor takes
some decision because of missing values and estimates default values these
missing data.}

\item{pcr_replicates}{a number indicating the number of pcr replicates or a
\code{character} vector with the name of the PCR replicates.}

\item{groups}{a \code{character} vector of motu group names for which the
motu design have to be repeated.}
}
\description{
motu names are build by concatenating a group name (facutative), a motu
set id, a motu number in the set and a PCR repeat id (facutative).
}
\examples{
  build_motu_names(5)

  build_motu_names(c(5, 3))

  build_motu_names(c(SAMP=5, CTRL=3), ndigits = 2)

  build_motu_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = 3)

  build_motu_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = c("A","B","C"))

  build_motu_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = c("A","B","C"),
                     groups=c("S1","S2"))

}
\author{
Eric Coissac <eric.coissac@metabarcoding.org>
}
