% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robisample.R
\name{build_sample_names}
\alias{build_sample_names}
\title{Builds a serie of sample names.}
\usage{
build_sample_names(
  n,
  ndigits = max(floor(log10(n)) + 1),
  pcr_replicates = NULL,
  sep = "_",
  groups = c(),
  verbose = is_robi_verbose()
)
}
\arguments{
\item{n}{an \code{integer} vector indicating the number of sample names to
generate per category. If \code{n} has names, they are used as category
names. Otherwhise category names are generated by concatenating the word
\code{sample} to a serie of letters.}

\item{ndigits}{a singme integer value indicating on how many digits the sample
numerical serie number have to be padded with 0s.}

\item{pcr_replicates}{a number indicating the number of pcr replicates or a
\code{character} vector with the name of the PCR replicates.}

\item{sep}{a character string to separate the name parts}

\item{groups}{a \code{character} vector of sample group names for which the
sample design have to be repeated.}

\item{verbose}{if `TRUE` warnings are emitted if the constructor takes
some decision because of missing values and estimates default values these
missing data.}
}
\description{
Sample names are build by concatenating a group name (facutative), a sample
set id, a sample number in the set and a PCR repeat id (facutative).
}
\examples{
  build_sample_names(5)

  build_sample_names(c(5, 3))

  build_sample_names(c(SAMP=5, CTRL=3), ndigits = 2)

  build_sample_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = 3)

  build_sample_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = c("A","B","C"))

  build_sample_names(c(SAMP=5, CTRL=3), ndigits = 2,
                     pcr_replicates = c("A","B","C"),
                     groups=c("S1","S2"))

}
\author{
Eric Coissac <eric.coissac@metabarcoding.org>
}
