% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_sample_ids.R
\name{left_parse_sample_ids}
\alias{left_parse_sample_ids}
\alias{left_parse_sample_ids.robidata}
\alias{right_parse_sample_ids.robisample}
\alias{left_parse_sample_ids.robimetabar}
\alias{right_parse_sample_ids.robimetabar}
\title{Parse the column containing sample ids}
\usage{
left_parse_sample_ids(
  object,
  parts,
  n,
  pattern = "_",
  category_mapping,
  sample_categories = "category",
  include_full = FALSE,
  replace = TRUE,
  prefix = "Left"
)

\method{left_parse_sample_ids}{robidata}(
  object,
  parts,
  n,
  pattern = "_",
  category_mapping,
  sample_categories = "category",
  include_full = FALSE,
  replace = TRUE,
  prefix = "Left"
)

\method{right_parse_sample_ids}{robisample}(
  object,
  parts,
  n,
  pattern = "_",
  category_mapping,
  sample_categories = "category",
  include_full = FALSE,
  replace = TRUE,
  prefix = "Right"
)

\method{left_parse_sample_ids}{robimetabar}(
  object,
  parts,
  n,
  pattern = "_",
  category_mapping,
  sample_categories = "category",
  include_full = FALSE,
  replace = TRUE,
  prefix = "Left"
)

\method{right_parse_sample_ids}{robimetabar}(
  object,
  parts,
  n,
  pattern = "_",
  category_mapping,
  sample_categories = "category",
  include_full = FALSE,
  replace = TRUE,
  prefix = "Right"
)
}
\arguments{
\item{object}{the object to parse a \code{\link[ROBITools2]{robisample}},
or a \code{\link[ROBITools2]{robimetabar}} instance.}

\item{parts}{a string vector indicating the name of each sub-pattern
produced. It can include or not a name for the last sub-pattern,
that actually corresponds to the full id. if the \code{include_full}
parameter is set to \code{FALSE} don't provide name for the full id.}

\item{n}{an integer value, used if no \code{parts} are specified. The
parts names will be build by pasting the \code{prefix} with a
number ranging from 1 to \code{n}.}

\item{pattern}{a string used to split the id. It cannot be a regular
pattern. It as to be a simple string.}

\item{category_mapping}{a category mapping object mapping actual category
names to the levels of a \code{\link[ROBITools2]{robicategory}}
instance (see \code{\link[ROBITools2]{robicategory_mapping}}.}

\item{sample_categories}{a string indicating which part of the id contains
the sample}

\item{include_full}{a logical value indicating if the full id has to
be conserved as a new field in the \code{\link[ROBITools2]{robisample}}.}

\item{replace}{if the initial \code{\link[ROBITools2]{robisample}} had columns
with the same names, do the new ones replace the originals.}

\item{prefix}{a string used as prefix when the \code{parts} parameter is missing.}
}
\value{
a \code{\link[ROBITools2]{robisample}} instance
}
\description{
Often the sample ids are composed of sevral parts
giving some information about the sample. Is it a
true sample, a control, or a blanck. To which PCR
replicate it belongs.
}
\details{
The parsing consists in spliting the id by a pattern
and to compose sub-ids by concatenating the firsts
(\code{left_parse_sample_ids}) or the last
(\code{right_parse_sample_ids}) of the intial id.
With an id looking like \code{P1_P2_P3} and a pattern
sets to underscore,  \code{left_parse_sample_ids} will generate
patterns \code{P1}, \code{P1_P2}, and \code{P1_P2_P3},
when \code{right_parse_sample_ids} will produce
\code{P3}, \code{P2_P3} and \code{P1_P2_P3}.
}
\examples{

    i <- c("blank_01_R1",   "blank_01_R2",
           "coExt_01_R1",   "coExt_01_R2",
           "PCRNeg_02_R1",  "PCRNeg_02_R2",
           "s_0001_R1",     "s_0002_R1",
           "s_0001_R2",     "s_0002_R2"  )

   s <- robisample(i)
   s \%>\% left_parse_sample_ids(parts = c("type","samples"),
                               include_full = FALSE)

   s \%>\% left_parse_sample_ids(parts = c("category","samples"),
                               category_mapping = robicategory_mapping(),
                               sample_categories = "category",
                               include_full = FALSE)

   s \%>\% right_parse_sample_ids(parts = c("type","samples"),
                               include_full = FALSE)

}
