% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alignment.R
\name{robi4mer}
\alias{robi4mer}
\alias{new_robi4mer}
\alias{is_robi4mer}
\alias{as_robi4mer}
\title{Vector of 4mers for multiplexing PCR.}
\usage{
robi4mer(length = 0)

new_robi4mer(
  object,
  adapte_length = TRUE,
  ...,
  verbose = is_robi_verbose(),
  class = character()
)

is_robi4mer(object)

as_robi4mer(object)
}
\arguments{
\item{length}{A non-negative integer specifying the desired length. Double
values will be coerced to integer: supplying an argument of length other
than one is an error.}

\item{object}{the object to convert.}

\item{adapte_length}{if \code{TRUE} longuest 4mers are shortened by trimming them
on the left side.}

\item{...}{Passed on to \code{\link[base:structure]{base::structure()}}. It allows for defining
attributes, specified in 4mer = value form, which will be attached to data
(see \code{\link{robiatomic}}).}

\item{verbose}{if \code{TRUE} warnings and messages are emitted when the
constructor takes some decision because of missing values and estimates
default values these missing data.}

\item{class}{Subclasses to assign to the new object, default: none}

\item{`object`}{the object to test.}
}
\value{
the \code{robi4mer()}and \code{new_robi4mer()} functions return a \code{robi4mer}
instance.

the \code{is_robi4mer()}, \code{is_robi4mer_forward()}, and \code{is_robi4mer_reverse()}
functions return a \code{\link{logical}} value.

the \code{as_robi4mer()} funtion returns a \code{robi4mer} value.
}
\description{
A \code{robi4mer} is character vector that is for storing short oligo nucleotides
used to 4mer sample. 4mers are always represented as uppercase letters and
contain only A, C, G, and T and every 4mer
}
\details{
The \code{robi4mer()} constructor builds a default \code{robi4mer} instance of
length defined by the \code{length} argument and composed of 4mers of length
zero.

The contructor \code{new_robi4mer} is the main function to create a
\code{robi4mer} instance. For simpler conversion to that class, it can be easier
to use the \code{as_robi4mer()} utility function.

\code{as_robi4mer} converts an object to a \code{robi4mer} instance.
}
\examples{

rt <- robi4mer(10)


rt <- new_robi4mer(c("ATC", "ATG", "CCC", "CTG"))


is_robi4mer(rt)
is_robi4mer("toto")


x <- c("AGTT","GGCT","CCGT","GATA","CCCC")
rt <- as_robi4mer(x)

}
\author{
Eric Coissac \href{mailto:eric.coissac@metabarcoding.org}{eric.coissac@metabarcoding.org}
}
