% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/robicategory.R
\name{robicategory}
\alias{robicategory}
\alias{new_robicategory}
\alias{as_robicategory}
\alias{levels <- .robicategory}
\title{Robicateroy vector to classify PCRs in a DNA metbarcoding experiment.}
\usage{
robicategory(length = 0)

new_robicategory(object, ..., verbose = is_robi_verbose(), class = character())

as_robicategory(object)

`levels  .robicategory`(x) <- value
}
\arguments{
\item{length}{A non-negative integer specifying the desired length. Double
values will be coerced to integer: supplying an argument of length other
than one is an error.}

\item{object}{the object to convert}

\item{...}{Passed on to \code{\link[base:structure]{base::structure()}}. It allows for
defining attributes, specified in tag = value form, which will be attached
to data (see \code{\link{robiatomic}}).}

\item{verbose}{if \code{TRUE} warnings and messages are emitted when the
constructor takes some decision because of missing values and estimates
default values these missing data.}

\item{class}{Subclasses to assign to the new object, default: none}

\item{x}{a \code{robicategory} instance}

\item{value}{A valid value for levels, actually not used.}
}
\value{
a \code{robicategory} instance

a \code{robicategory} instance
}
\description{
In the \link{ROBITools2} library every PCR referenced in a \code{\link{robisample}}
have to be associated to a category. The set of categories
is predefined and cannot be changed by the user. If users need another more
precise classification system of their PCR they can us a standard \code{\link[base:factor]{base::factor}}
with their own defined levels.
}
\details{
The \code{robicategory} system define five categories:
\itemize{
\item \strong{sample}         : which corresponds to PCR run on DNA corresponding to a true sample.
\item \strong{positive_ctrl}  : for PCRs corresponding to the amplification of a
positive control (mock community).
\item \strong{extraction_blk} : for PCRs corresponding to an extraction control.
\item \strong{pcr_blk}        : for PCRs corresponding to a PCR control (no DNA template).
\item \strong{sequencing_blk} : for combinaisons of tags corresponing to no PCR
}

A \code{robicategory} is implemented as a factor. \code{NA} values are not allowed in \code{robicategory}.

The \code{robicategory()} constructor builds a default \code{robicategory} instance of length
defined by the \code{length} argument and composed of elements corresponding to the \code{sample}
modality.

The contructor \code{new_robicategory} is the main function to
create a \code{robicategory} instance. For simpler conversion to that class,
it can be easier to use the \code{as_robicategory()} utility function.

\code{as_robiatomic} converts an object to a \code{robicategory} instance.

This implementation of the \code{\link[base]{levels}} method for
the \code{robicategory} just allows for substituting levels of a
\code{robicategory} by themselves.
}
\examples{

rc <- robicategory(10)


rc <- new_robicategory(rep(c("sample", "positive_ctrl", "pcr_blk"),
                           c(3, 2, 1)))


x <- factor(rep(c("sample", "positive_ctrl", "pcr_blk"),
            c(3, 2, 1)))
as_robicategory(x)


x <- robicategory(10)
levels(x) <- c("sample", "positive_ctrl",
               "extraction_blk", "pcr_blk",
               "sequencing_blk")

\dontrun{
levels(x) <- c("A", "B", "C", "D")
}

}
\author{
Eric Coissac \href{mailto:eric.coissac@metabarcoding.org}{eric.coissac@metabarcoding.org}
}
