% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_robitag.R
\name{robitag}
\alias{robitag}
\alias{new_robitag}
\alias{is_robitag}
\alias{as_robitag}
\alias{robitag_forward}
\alias{new_robitag_forward}
\alias{is_robitag_forward}
\alias{as_robitag_forward}
\alias{robitag_reverse}
\alias{new_robitag_reverse}
\alias{is_robitag_reverse}
\alias{as_robitag_reverse}
\title{Vector of tags for multiplexing PCR.}
\usage{
robitag(length = 0)

new_robitag(
  object,
  adapte_length = TRUE,
  ...,
  verbose = is_robi_verbose(),
  class = character()
)

is_robitag(object)

as_robitag(object)

robitag_forward(length = 0)

new_robitag_forward(
  object,
  adapte_length = TRUE,
  ...,
  verbose = is_robi_verbose(),
  class = character()
)

is_robitag_forward(object)

as_robitag_forward(object)

robitag_reverse(length = 0)

new_robitag_reverse(
  object,
  adapte_length = TRUE,
  ...,
  verbose = is_robi_verbose(),
  class = character()
)

is_robitag_reverse(object)

as_robitag_reverse(object)
}
\arguments{
\item{length}{A non-negative integer specifying the desired length. Double
values will be coerced to integer: supplying an argument of length other
than one is an error.}

\item{object}{the object to convert}

\item{adapte_length}{if \code{TRUE} longuest tags are shortened by trimming them
on the left side.}

\item{...}{Passed on to \code{\link[base:structure]{base::structure()}}. It allows for defining
attributes, specified in tag = value form, which will be attached to data
(see \code{\link{robiatomic}}).}

\item{verbose}{if \code{TRUE} warnings and messages are emitted when the
constructor takes some decision because of missing values and estimates
default values these missing data.}

\item{class}{Subclasses to assign to the new object, default: none}

\item{`object`}{the object to test.}
}
\value{
the \code{robitag()}and \code{new_robitag()} functions return a \code{robitag}
instance.

the \code{is_robitag()}, \code{is_robitag_forward()}, and \code{is_robitag_reverse()}
functions return a \code{\link{logical}} value.

the \code{as_robitag()} funtion returns a \code{robitag} value.

the \code{robitag_forward()}and \code{new_robitag_forward()} functions return a
\code{robitag_forward} instance.

the \code{as_robitag_forward()} funtion returns a \code{robitag_forward} value.

the `robitag_reverse()`and `new_robitag_reverse()` functions
  return a `robitag_reverse` instance.

the \code{as_robitag_reverse()} funtion returns a \code{robitag_reverse} value.
}
\description{
A \code{robitag} is character vector that is for storing short oligo nucleotides
used to tag sample. Tags are always represented as uppercase letters and
contain only A, C, G, and T and every tag
}
\details{
The \code{robitag()} constructor builds a default \code{robitag} instance of
length defined by the \code{length} argument and composed of tags of length
zero.

The contructor \code{new_robitag} is the main function to create a
\code{robitag} instance. For simpler conversion to that class, it can be easier
to use the \code{as_robitag()} utility function.

\code{as_robitag} converts an object to a \code{robitag} instance.

\code{robitag_forward} is a specialisation of the class \code{robitag}. When
PCR products are tagged on both extremities, that class allows for
distinguising tags on the 5' end of the PCR product (\code{robitag_forward})
from those at the 3' end extremity (\code{reverse_tag}). Except that semantic
distinction no difference exists with \code{robitag}.

\code{as_robitag} converts an object to a \code{robitag} instance.

`robitag_reverse` is a specialisation of the class `robitag`.
   It plays the same role than `robitag_forward` but for the reverse
   tag located at the three prime end of the PCR product.

\code{is_robitag_reverse} tests if an object belongs the \code{is_robitag_reverse} class

\code{as_robitag} converts an object to a \code{robitag} instance.
}
\examples{

rt <- robitag(10)


rt <- new_robitag(c("ATC", "ATG", "CCC", "CTG"))


is_robitag(rt)
is_robitag("toto")


x <- c("AGT","GGT","CGT","GTA","CCC")
rt <- as_robitag(x)


rf <- new_robitag_forward(c("AGT","GGT","CGT","GTA","CCC"))


is_robitag_forward(rf)
is_robitag_forward("toto")

x <- c("AA","CC","CG")
as_robitag_forward(x)


rr <- new_robitag_reverse(c("AGT","GGT","CGT","GTA","CCC"))


is_robitag_reverse(rr)
is_robitag_reverse("toto")

x <- c("AA","CC","CG")
as_robitag_reverse(x)

}
\author{
Eric Coissac \href{mailto:eric.coissac@metabarcoding.org}{eric.coissac@metabarcoding.org}
}
