% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_robiobject.R, R/type_robiatomic.R,
%   R/alignment.R, R/robidata.R, R/type_robitaxid.R, R/type_robitag.R,
%   R/type_robiuniqueid.R, R/robimotu.R, R/robicategory.R, R/robisample.R,
%   R/robimetabar.R, R/robimutation.R
\name{validate_robiobject}
\alias{validate_robiobject}
\alias{validate_robiobject.default}
\alias{is_a_valid_robiobject}
\alias{validate_robiobject.robiatomic}
\alias{validate_robiobject.robi4mer}
\alias{validate_robiobject.robidata}
\alias{validate_robiobject.robitaxid}
\alias{validate_robiobject.robitag}
\alias{validate_robiobject.robiuniqueid}
\alias{validate_robiobject.robimotu}
\alias{validate_robiobject.robicategory}
\alias{validate_robiobject.robisample}
\alias{validate_robiobject.robimetabar}
\alias{validate_robiobject.robimutation}
\title{Validates a new object}
\usage{
validate_robiobject(object, verbose = is_robi_verbose())

\method{validate_robiobject}{default}(object, verbose = is_robi_verbose())

is_a_valid_robiobject(object)

\method{validate_robiobject}{robiatomic}(object, verbose = is_robi_verbose())

\method{validate_robiobject}{robi4mer}(object)

\method{validate_robiobject}{robidata}(object, verbose = is_robi_verbose())

\method{validate_robiobject}{robitaxid}(object)

\method{validate_robiobject}{robitag}(object)

\method{validate_robiobject}{robiuniqueid}(object, verbose = is_robi_verbose())

\method{validate_robiobject}{robimotu}(object, verbose = is_robi_verbose())

\method{validate_robiobject}{robicategory}(object)

\method{validate_robiobject}{robisample}(object, verbose = TRUE)

\method{validate_robiobject}{robimetabar}(object, verbose = is_robi_verbose())

\method{validate_robiobject}{robimutation}(object)
}
\arguments{
\item{object}{the instance to test}

\item{verbose}{a logical indicating if some messages
have to be printed during the checks.}
}
\value{
the input object

a logical value
}
\description{
Methods \code{validate_robiobject} chek that all the
invariants on an object are respected. If not
an error is raised, otherwise the object is returned

The function returns \code{TRUE} if the object is valid \code{FALSE} otherwise.
}
\details{
The default version only returns the
    checked object without any test

The \code{\link{robiatomic}} version of that method
ckecks that the instance is an \code{\link[rlang:type-predicates]{atomic}} object.

The \code{\link{robi4mer}} version of that method ckecks that :
\itemize{
\item All the 4mers have the same length
\item Every 4mers are composed only of upper case A, C, G, or T
}

The \code{\link[ROBITools2]{robidata}} version
    ckecks that the instance contains a one and only one
    \code{\link[ROBITools2]{robiuniqueid}} instance.

The \code{\link{robitag}} version of that method ckecks that :
\itemize{
\item All the tags have the same length
\item Every tags are composed only of upper case A, C, G, or T
}

The \code{\link[ROBITools2]{robiuniqueid}} version
    ckecks that the instance contains no duplicated identifers.

The \code{\link[ROBITools2]{robimotu}} version
    ckecks that if \code{\link[ROBITools2]{robitaxid}} columns
    are present that at least one, but no more, is a
    \code{\link[ROBITools2]{robitaxid_master}} instance.

The \code{\link{robicategory}} version of that method ckecks that :
\itemize{
\item the instance does not contain any \code{NA} value.
\item the levels of the instance corresponds to the predefined levels
}

The \code{\link[ROBITools2]{robisample}} version
    ckecks that a single column of.\code{\link[ROBITools2]{robicategory}}
    exist, and if tags columns are present it must be a single
    \code{\link[ROBITools2]{robitag}} or a single couple of
    \code{\link[ROBITools2]{robitag_forward}} and
    \code{\link[ROBITools2]{robitag_reverse}}
    columns

The \code{robimetabar} version of this methode checks that
taxonomy declared as full verified that constaint.
}
\examples{
   x <- new_robiuniqid(c("A","B","C"))
   validate_robiobject(x)
   x <- new_robiuniqid(c("A","B","C"))
   is_a_valid_robiobject(x)

xd <- new_robiatomic(1:10,class="robifoo")
validate_robiobject(xd)


rt <- new_robi4mer(c("AGT","GGT","CGT","GTA","CCC"))
validate_robiobject(rt)


rt <- new_robitag(c("AGT","GGT","CGT","GTA","CCC"))


# For robimotu instance
s <- robimotu(n=20)
validate_robiobject(s)


rc <- robicategory(10)
validate_robiobject(xd)


# Validation of a robisample
s <- robisample(n=20)
validate_robiobject(s)

}
\author{
Eric Coissac <eric.coissac@metabarcoding.org>
}
