% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_obifasta.R
\name{write_obifasta}
\alias{write_obifasta}
\title{Writes a tibble containing sequences in a fasta file.}
\usage{
write_obifasta(
  data,
  file = NULL,
  make_unique = TRUE,
  verbose = is_robi_verbose()
)
}
\arguments{
\item{data}{the \code{tibble} to format}

\item{file}{a writable \code{connection} or a character string naming the
file to write to, or NULL to print on stdout.}

\item{make_unique}{insure that the ids of the sequences are unique}

\item{verbose}{if \code{TRUE} warnings and messages are emitted when the
function takes some decision.}
}
\description{
To be writable as a fasta file a \code{tibble} requires to have
a sequence column containing the \code{character} strings. The
other columns that can be used during the formating are:
}
\details{
\itemize{
\item \code{id} : used as sequence id located just after the \code{>}
sign,
\item \code{definition} : which is added at the end of the title line,
\item \code{features} : the features must be a \code{tibble} or a \code{data.frame},
each column of the table will be translated to
a \emph{key}, \emph{value} feature following the extended
obifasta format used by \emph{OBITools}
}
}
\examples{
   filename <- system.file("extdata", "B5.fasta.gz",
                           package = "ROBITools2")

   seqs <- read_obifasta(filename)
   write_obifasta(seqs)

}
