#include <R.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include <stdint.h>
#include "robidnacode.h"


#define ABS(X) (((X) >= 0) ? (X):-(X))
#define MIN(X,Y) (((X) < (Y)) ? (X):(Y))
#define MAX(X,Y) (((X) > (Y)) ? (X):(Y))


static int32_t is_d0_or_d1(const char *seq1, const char* seq2, size_t length1, size_t length2) {
  const char  *b1, *e1;
  const char  *b2, *e2;
  int32_t   code;
  int32_t pos1, pos2, pos;

  // First case length difference > 1

  if (ABS((int32_t)length1 - (int32_t)length2) > 1)
    return 0;

  b1 = seq1;
  b2 = seq2;
  e1 = seq1 + length1;
  e2 = seq2 + length2;


  // Scans the sequences from their beginings as long as they are identical

  while(*b1 == *b2 && (*b1 != 0 && *b2 != 0)) {
    b1++;
    b2++;
  }

  if ((*b1 == 0) && (*b2 == 0)) return 256;

  // Scans the sequences from their ends as long as they are identical

  while(*e1 == *e2 && (e1 > seq1 || e2 > seq2)) {
    e1--;
    e2--;
  }

 if ((length1 == length2 && (e1 > b1 || e2 > b2)) |
     (length1 > length2  && e1 > b1) ||
     (length1 < length2  && e2 > b2))
    return 0;

  pos1 = e1 - seq1;
  pos2 = e2 - seq2;
  pos = MIN((b1>= e1) ? MAX(pos1,pos2):0,INT_LEAST16_MAX);

  if (pos2 > pos1)
    code = (robi_codeDNA[*e2]);
  if (pos2 < pos1)
    code = (robi_codeDNA[*e1]*16);
  if (pos2 == pos1)
    code = (robi_codeDNA[*e1]*16) + (robi_codeDNA[*e2]);

  code+= 512 + (pos * 65536);

  return code;
}


SEXP ROBI_is_d0_or_d1(SEXP sequences1,SEXP sequences2,SEXP length1,SEXP length2) {
  const char *csource1, *csource2;
  size_t l1, l2;
  size_t nseq;
  size_t i;

  SEXP results;
  SEXP svalue1;
  SEXP svalue2;

  if (! isString(sequences1))
    error("the sequences1 parameter must be a character vector");

  if (! isString(sequences2))
    error("the sequences2 parameter must be a character vector");

  if (! isInteger(length1))
    error("the length1 parameter must be an integer vector");

  if (! isInteger(length2))
    error("the length2 parameter must be an integer vector");

  nseq   = LENGTH(sequences1);

  if ( nseq != LENGTH(sequences2) || nseq != LENGTH(length1) || nseq != LENGTH(length2))
    error("sequence21, sequences2, length1, and length2 must have the same length");


  results = PROTECT(NEW_INTEGER(nseq));

  for (i = 0; i < nseq; i++) {
    svalue1 = STRING_ELT(sequences1,i);
    svalue2 = STRING_ELT(sequences2,i);

    if (svalue1 != NA_STRING && svalue2 != NA_STRING) {
      csource1 = CHAR(svalue1);
      csource2 = CHAR(svalue2);
      INTEGER(results)[i] = is_d0_or_d1(CHAR(svalue1),CHAR(svalue2),
                                        INTEGER(length1)[i],INTEGER(length2)[i]);
    }
    else
      INTEGER(results)[i] = NA_INTEGER;
  }

  UNPROTECT(1);
  return results;
}


