#include <R.h>
#include <Rinternals.h>
#include <Rdefines.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdint.h>
#include <string.h>
#include <stdbool.h>
#include <limits.h>

static int32_t dnadist(const char* sequence1,
                       const char* sequence2,
                       size_t lseq1,
                       size_t lseq2,
                       int32_t* buffer);

int32_t dnadist(const char* sequence1,
                const char* sequence2,
                size_t lseq1,
                size_t lseq2,
                int32_t* buffer) {
  int32_t *dist_mat, *dist_p;
  int32_t *path_mat, *path_p;
  int32_t *best_jmp_v;
  int32_t *best_jmp_h;

  size_t  mat_bytes_size;
  int32_t line_size;
  size_t i,j;

  int32_t smin,snext,pmin,pd;


  line_size = lseq1 + 1;
  mat_bytes_size = line_size * (lseq2 + 1) ;

  dist_mat     = buffer;
  path_mat     = dist_mat   + mat_bytes_size;
  best_jmp_v   = path_mat   + mat_bytes_size;
  best_jmp_h   = best_jmp_v + line_size;

  memset(dist_mat,
         0,
         (mat_bytes_size * 2  + line_size + lseq2 + 1) * sizeof(int32_t));

  dist_p = dist_mat;
  path_p = path_mat;

  //
  // First cell
  //
  dist_p++;
  path_p++;

  //
  // First line
  //

  for (i = 1 ; i <= lseq1; i++) {
    *dist_p = 1;
    *path_p = 1;

    dist_p++;
    path_p++;
  }

  //
  // The other lines
  //

  for (j = 1; j <= lseq2; j++) {

    // First cell of each line

    *dist_p = 1;
    *path_p = -1;

    dist_p++;
    path_p++;

    for (i = 1 ; i <= lseq1; i++) {
      // Short vertical
      smin = dist_p[-line_size] + (path_p[-line_size]==-1 ? 0:1);
      pmin = -1;

      // Short horizontal
      snext = dist_p[-1] + (path_p[-1]==1 ? 0:1);
      if (snext <= smin) {
        smin = snext;
        pmin = 1;
      }


      // Diagonal
      pd =  (sequence1[i-1]==sequence2[j-1]) ? 0:1;
      snext = dist_p[-line_size - 1] + (path_p[-line_size - 1]==0 ? 0:pd);
      if (snext <= smin) {
        smin = snext;
        pmin = 0;
      }

      *dist_p = smin;
      *path_p = pmin;

      dist_p++;
      path_p++;
    }
  }

  return *(dist_p - 1);
}

SEXP ROBI_dnadist(SEXP sequences1,SEXP sequences2,SEXP length1,SEXP length2) {
  size_t nseq;
  size_t i;
  size_t lmax;
  size_t l1, l2;



  const char *csource1,*csource2;

  int32_t *address;

  SEXP distances;
  SEXP svalue1;
  SEXP svalue2;


  if (! isString(sequences1))
    error("the sequences1 parameter must be a character vector");

  if (! isString(sequences2))
    error("the sequences2 parameter must be a character vector");

  if (! isInteger(length1))
    error("the length1 parameter must be an integer vector");

  if (! isInteger(length2))
    error("the length2 parameter must be an integer vector");

  nseq   = LENGTH(sequences1);

  if ( nseq != LENGTH(sequences2) || nseq != LENGTH(length1) || nseq != LENGTH(length2))
    error("sequence1, sequences2, length1, and length2 must have the same length");

  lmax = 0;

  for (i = 0 ; i < nseq; i++) {
    if (INTEGER(length1)[i] > lmax) lmax = INTEGER(length1)[i];
    if (INTEGER(length2)[i] > lmax) lmax = INTEGER(length2)[i];
  }

  address = (int32_t*)R_alloc((lmax + 1) * (lmax + 2) * 2 ,sizeof(int32_t));

  distances  = PROTECT(NEW_INTEGER(nseq));

  for (i = 0; i < nseq ; i++) {
	  svalue1 = STRING_ELT(sequences1,i);
    svalue2 = STRING_ELT(sequences2,i);

    if (svalue1 != NA_STRING && svalue2 != NA_STRING) {
      csource1 = CHAR(svalue1);
      csource2 = CHAR(svalue2);
      l1 = INTEGER(length1)[i];
      l2 = INTEGER(length2)[i];

      INTEGER(distances)[i]  =  dnadist(csource1, csource2,
                                        l1, l2,
                                        address);
    }
    else {
      INTEGER(distances)[i]  =  NA_INTEGER;
    }
  }

  UNPROTECT(1);
  return distances;
}
