#include <R.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include <stdint.h>

#include "__sse.h"

#define IS_BIG_ENDIAN (*(uint16_t *)"\0\xff" < 0x100)

inline static uchar_v hash4m128(uchar_v frag)
{
	uchar_v words;

	vUInt8 mask_03= _MM_SET1_EPI8(0x03);        // charge le registre avec 16x le meme octet
	vUInt8 mask_FC= _MM_SET1_EPI8(0xFC);

	frag.m = _MM_SRLI_EPI64(frag.m,1);         // shift logic a droite sur 2 x 64 bits
	frag.m = _MM_AND_SI128(frag.m,mask_03);    // and sur les 128 bits


	words.m= _MM_SLLI_EPI64(frag.m,2);
	words.m= _MM_AND_SI128(words.m,mask_FC);
	frag.m = _MM_SRLI_SI128(frag.m,1);
	words.m= _MM_OR_SI128(words.m,frag.m);

	words.m= _MM_SLLI_EPI64(words.m,2);
	words.m= _MM_AND_SI128(words.m,mask_FC);
	frag.m = _MM_SRLI_SI128(frag.m,1);
	words.m= _MM_OR_SI128(words.m,frag.m);

	words.m= _MM_SLLI_EPI64(words.m,2);
	words.m= _MM_AND_SI128(words.m,mask_FC);
	frag.m = _MM_SRLI_SI128(frag.m,1);
	words.m= _MM_OR_SI128(words.m,frag.m);

	return words;
}

#ifdef __SSE2__

inline static int anyzerom128(vUInt8 data)
{
	vUInt8 mask_00= _MM_SETZERO_SI128();
	uint64_v tmp;
	tmp.m = _MM_CMPEQ_EPI8(data,mask_00);
	return (int)(tmp.c[0]!=0 || tmp.c[1]!=0);
}

#else

inline static int anyzerom128(vUInt8 data)
{
	int i;
	um128 tmp;
	tmp.i = data;
	for (i=0;i<8;i++)
		if (tmp.s8[i]==0)
			return 1;
	return 0;
}

#endif

inline static void dumpm128(unsigned short *table,vUInt8 data)
{
	memcpy(table,&data,16);
}

/**
 * Compute 4mer occurrence table from a DNA sequence
 *
 *	sequence : a pointer to the null terminated nuc sequence
 *	table    : a pointer to a 256 cells unisgned char table for
 *	           storing the occurrence table
 *	count    : pointer to an int value used as a return value
 *	           containing the global word counted
 *
 *	returns the number of words observed in the sequence with a
 *	count greater than 255.
 */

static int buildTable(const char* sequence, unsigned char *table, size_t nseq, int *count)
{
	int overflow = 0;
	int wc=0;
	int i;
  size_t j;
	vUInt8 mask_00= _MM_SETZERO_SI128();

	uchar_v frag;
	uchar_v words;
	uchar_v zero;

	char* s;

	s=(char*)sequence;

	// encode ascii sequence with  A : 00 C : 01  T: 10   G : 11

	for(frag.m=_MM_LOADU_SI128((vUInt8*)s);
		! anyzerom128(frag.m);
		s+=12,frag.m=_MM_LOADU_SI128((vUInt8*)s))
	{
		words= hash4m128(frag);

	//    printf("%d %d %d %d\n",words.c[0],words.c[1],words.c[2],words.c[3]);
    j = words.c[0] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[1] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[2] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[3] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[4] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[5] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[6] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[7] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[8] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[9] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[10] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;
    j = words.c[11] * nseq;
		if (table[j]<255)  table[j]++;  else overflow++;

		wc+=12;
	}

	zero.m=_MM_CMPEQ_EPI8(frag.m,mask_00);
	//printf("frag=%d %d %d %d\n",frag.c[0],frag.c[1],frag.c[2],frag.c[3]);
	//printf("zero=%d %d %d %d\n",zero.c[0],zero.c[1],zero.c[2],zero.c[3]);
	words = hash4m128(frag);

	if (zero.c[0]+zero.c[1]+zero.c[2]+zero.c[3]==0)
		for(i=0;zero.c[i+3]==0;i++,wc++) {
      j = words.c[i] * nseq;
			if (table[j]<255) table[j]++;  else overflow++;
		}

	if (count) *count=wc;
	return overflow;
}

static inline vUInt16 partialminsum(vUInt8 ft1,vUInt8 ft2)
{
	vUInt8   mini;
	vUInt16  minilo;
	vUInt16  minihi;
	vUInt8 mask_00= _MM_SETZERO_SI128();

	mini      = _MM_MIN_EPU8(ft1,ft2);
	minilo    = _MM_UNPACKLO_EPI8(mini,mask_00);
	minihi    = _MM_UNPACKHI_EPI8(mini,mask_00);

	return _MM_ADDS_EPU16(minilo,minihi);
}

/**
 * Recode an unsigned integer smaller than 16,777,216
 * to insure no zero byte inside.
 */
static inline uint32_t encode_non_zero_int(uint32_t value) {
  if (value >= 0x00FFFFFF)
    return 0xFFFFFFFF;

  return  ((value & 0x00800000) << 3) | \
          ((value & 0x00008000) << 10) | \
          ((value & 0x00000080) << 17) | \
          ((value & 0x007F7F7F) | 0x80808080);
}

static inline uint32_t decode_non_zero_int(uint32_t value) {
 return ((value & 0x04000000) >> 3) | \
        ((value & 0x02000000) >> 10) | \
        ((value & 0x01000000) >> 17) | \
        (value  & 0x007F7F7F);
}

/**
 * Computes the maximum number of shared 4mers between both the sequences.
 *
 * If the overflow is equal to 0 for at least one sequence, the count is exact.
 * If bot the overflow are non-zero then the returned value is an upperbond.
 *
 */
static int32_t compareTable(unsigned char *t1, int32_t over1, unsigned char* t2,  int32_t over2)
{
	vUInt8   ft1;
	vUInt8   ft2;
	vUInt8  *table1=(vUInt8*)t1;
	vUInt8  *table2=(vUInt8*)t2;
	ushort_v summini;
	size_t   i;
	int32_t  total;

	ft1 = _MM_LOADU_SI128(table1);
	ft2 = _MM_LOADU_SI128(table2);
	summini.m = partialminsum(ft1,ft2);
	table1++;
	table2++;


	for (i=1;i<16;i++,table1++,table2++)
	{
		ft1 = _MM_LOADU_SI128(table1);
		ft2 = _MM_LOADU_SI128(table2);
		summini.m = _MM_ADDS_EPU16(summini.m,partialminsum(ft1,ft2));
	}

	// Finishing the sum process

	summini.m = _MM_ADDS_EPU16(summini.m,_MM_SRLI_SI128(summini.m,8)); // sum the 4 firsts with the 4 lasts
	summini.m = _MM_ADDS_EPU16(summini.m,_MM_SRLI_SI128(summini.m,4));

	total = summini.c[0]+summini.c[1];
	total+= (over1 < over2) ? over1:over2;

	return total;
}

SEXP ROBI_4mer_table(SEXP sequences) {
  unsigned char* buffer;
  unsigned char  swap1;
  unsigned char  swap2;
  const char* csource;
  size_t nseq;
  size_t i;
  size_t j;

  SEXP srep;
  SEXP hashes;
  SEXP svalue;
  SEXP soverflow;
  SEXP sn4mer;
  SEXP sdim;

  int* overflow;
  int* n4mer;

  if (! isString(sequences))
    error("the sequences parameter must be a character vector");

  srep = PROTECT(NEW_LIST(3));

  nseq   = LENGTH(sequences);
  hashes = PROTECT(NEW_RAW(nseq * 256));
  memset(RAW(hashes),0,nseq * 256 * sizeof(unsigned char));

  soverflow = PROTECT(NEW_INTEGER(nseq));
  sn4mer = PROTECT(NEW_INTEGER(nseq));
  sdim = PROTECT(NEW_INTEGER(2));
  INTEGER(sdim)[0] = nseq;
  INTEGER(sdim)[1] = 256;
  SET_DIM(hashes,sdim);


  overflow = INTEGER(soverflow);
  n4mer = INTEGER(sn4mer);

  for (i = 0; i < nseq; i++) {
    buffer = RAW(hashes) + i;
    svalue = STRING_ELT(sequences,i);

    if (svalue != NA_STRING) {
      csource = CHAR(svalue);
      overflow[i] = buildTable(csource,buffer,nseq,n4mer + i);
    }
    else {
      overflow[i] = 0;
      n4mer[i] = 0;
      memset(buffer,0,256*sizeof(unsigned char));

    }
  }


  SET_VECTOR_ELT(srep,0,hashes);
  SET_VECTOR_ELT(srep,1,soverflow);
  SET_VECTOR_ELT(srep,2,sn4mer);


  UNPROTECT(5);
  return srep;
}


SEXP ROBI_max_common_4mer(SEXP x, SEXP y) {
  size_t nx, ny, si;
  size_t i, j;

  const char* cx;
  const char* cy;

  int32_t overx, overy;

  SEXP swap;
  SEXP vx;
  SEXP vy;
  SEXP result;

  if (! isString(x))
    error("the x parameter must be a character vector");

  if (! isString(y))
    error("the y parameter must be a character vector");

  nx = LENGTH(x);
  ny = LENGTH(y);

  if (ny > nx) {
    swap = x;
    x = y;
    y = swap;

    si = nx;
    nx = ny;
    ny = si;
  }

  result = PROTECT(NEW_INTEGER(nx));

  for (i = 0, j = 0; i < nx; i++, j++) {
    if (j == ny) j = 0;

    vx = STRING_ELT(x,i);
    vy = STRING_ELT(y,j);

    if(vx != NA_STRING && vy != NA_STRING) {
      cx = CHAR(vx);
      cy = CHAR(vy);

      //overx = overflow(cx);
      //overy = overflow(cy);

      INTEGER(result)[i] = compareTable((unsigned char*)cx, overx,
                                        (unsigned char*)cy, overy);
    }
    else
      INTEGER(result)[i] = NA_INTEGER;
  }

  UNPROTECT(1);
  return result;
}
