#include <R.h>
#include <Rinternals.h>
#include <Rdefines.h>
#include <stdint.h>
#include "robidnacode.h"

SEXP ROBI_as_chararacter_mutation(SEXP mutations) {
  size_t n;
  size_t i;
  int32_t m;
  const char* rep;
  char buffer[256];

  SEXP results;


  if (! isInteger(mutations))
    error("the mutations parameter must be an integer vector");

  n   = LENGTH(mutations);
  results = PROTECT(NEW_STRING(n));

  for (i = 0; i < n; i++) {
    m = INTEGER(mutations)[i];

    if (INTEGER(mutations)[i]==NA_INTEGER)
      SET_STRING_ELT(results,i,NA_STRING);
    else {
        if (m==0) rep = "[SEVERAL]";
        else {
          if (m & 256) rep = "[NO MUTATION]";
          else {
            snprintf(buffer,256,"[%5d : %c > %c]",
                     (m >> 16) + 1,
                     robi_DNAcode[(m & 240) >> 4],
                     robi_DNAcode[m&15]);
            rep = buffer;
          }
        }
        SET_STRING_ELT(results,i,mkChar((const char*)rep));
    }
  }

  UNPROTECT(1);
  return results;
}

SEXP ROBI_substitution_mutation(SEXP mutations) {
  size_t n;
  size_t i;
  int32_t m;
  char buffer[256];

  SEXP results;


  if (! isInteger(mutations))
    error("the mutations parameter must be an integer vector");

  n   = LENGTH(mutations);
  results = PROTECT(NEW_STRING(n));

  for (i = 0; i < n; i++) {
    m = INTEGER(mutations)[i];

    if (INTEGER(mutations)[i]==NA_INTEGER || m==0 || (m & 256))
      SET_STRING_ELT(results,i,NA_STRING);
    else {
        snprintf(buffer,256,"[%c > %c]", robi_DNAcode[(m & 240) >> 4], robi_DNAcode[m&15]);
        SET_STRING_ELT(results,i,mkChar((const char*)buffer));
    }
  }

  UNPROTECT(1);
  return results;
}

SEXP ROBI_origin_mutation(SEXP mutations) {
  size_t n;
  size_t i;
  int32_t m;
  char buffer[256];

  SEXP results;


  if (! isInteger(mutations))
    error("the mutations parameter must be an integer vector");

  n   = LENGTH(mutations);
  results = PROTECT(NEW_STRING(n));

  for (i = 0; i < n; i++) {
    m = INTEGER(mutations)[i];

    if (INTEGER(mutations)[i]==NA_INTEGER || m==0 || (m & 256))
      SET_STRING_ELT(results,i,NA_STRING);
    else {
        snprintf(buffer,256,"%c", robi_DNAcode[(m & 240) >> 4]);
        SET_STRING_ELT(results,i,mkChar((const char*)buffer));
    }
  }

  UNPROTECT(1);
  return results;
}

SEXP ROBI_to_mutation(SEXP mutations) {
  size_t n;
  size_t i;
  int32_t m;
  char buffer[256];

  SEXP results;


  if (! isInteger(mutations))
    error("the mutations parameter must be an integer vector");

  n   = LENGTH(mutations);
  results = PROTECT(NEW_STRING(n));

  for (i = 0; i < n; i++) {
    m = INTEGER(mutations)[i];

    if (INTEGER(mutations)[i]==NA_INTEGER || m==0 || (m & 256))
      SET_STRING_ELT(results,i,NA_STRING);
    else {
        snprintf(buffer,256,"%c", robi_DNAcode[m&15]);
        SET_STRING_ELT(results,i,mkChar((const char*)buffer));
    }
  }

  UNPROTECT(1);
  return results;
}

SEXP ROBI_position_mutation(SEXP mutations) {
  size_t n;
  size_t i;
  size_t pos;
  int32_t m;
  const char* rep;
  char buffer[256];

  SEXP results;


  if (! isInteger(mutations))
    error("the mutations parameter must be an integer vector");

  n   = LENGTH(mutations);
  results = PROTECT(NEW_INTEGER(n));

  for (i = 0; i < n; i++) {
    m = INTEGER(mutations)[i];

    if (INTEGER(mutations)[i]==NA_INTEGER || m==0 || (m & 256))
      INTEGER(results)[i]=NA_INTEGER;
    else
      INTEGER(results)[i]= (m >> 16) + 1;
  }

  UNPROTECT(1);
  return results;
}

